BlackmarketPreviewRotationControl = BlackmarketPreviewRotationControl or {}
BlackmarketPreviewRotationControl._mod_path = ModPath
BlackmarketPreviewRotationControl._settings_path = ModPath .. "menu/menu.json"
BlackmarketPreviewRotationControl._settings_save_path = SavePath .. "blackmarketpreviewrotationcontrol.txt"
BlackmarketPreviewRotationControl._localization_path = ModPath .. "loc/"
BlackmarketPreviewRotationControl.default_localization_file = "english.txt"
BlackmarketPreviewRotationControl.settings = {
	shaker_disabled = true,
	default_yaw = 1,
	default_pitch = 1,
	default_roll = 1,
	default_rotation_enabled = false,
	yaw_speed = 5,
	pitch_speed = 0,
	roll_speed = 0,
	rotation_acceleration_interval = 1,
	slow_stop = false --not yet implemented
}
BlackmarketPreviewRotationControl.params = nil

function BlackmarketPreviewRotationControl:IsShakerDisabled()
	return self.settings.shaker_disabled
end

function BlackmarketPreviewRotationControl:GetDefaultRotation()
	return Rotation(self.settings.default_yaw,self.settings.default_pitch,self.settings.default_roll)
end

function BlackmarketPreviewRotationControl:GetYawSpeed()
	if self.params then 
		return self.params.yaw_speed
	else
		return self.settings.yaw_speed
	end
end

function BlackmarketPreviewRotationControl:GetPitchSpeed()
	if self.params then 
		return self.params.pitch_speed
	else
		return self.settings.pitch_speed
	end
end

function BlackmarketPreviewRotationControl:GetRollSpeed()
	if self.params then 
		return self.params.roll_speed
	else
		return self.settings.roll_speed
	end
end

function BlackmarketPreviewRotationControl:IsAutomaticRotationEnabled()
	if self.params then 
		return self.params.rotation_enabled
	else
		return self.settings.default_rotation_enabled
	end
end

function BlackmarketPreviewRotationControl:GetRotationAcceleration(axis)
	--todo different axis acceleration rates (not used)
	if axis == "pitch" then 
		return self.settings.rotation_acceleration_interval
	elseif axis == "roll" then 
		return self.settings.rotation_acceleration_interval
	else --default to yaw
		return self.settings.rotation_acceleration_interval
	end
end

function BlackmarketPreviewRotationControl:Save()
	local file = io.open(self._settings_save_path, "w+")
	if file then
		file:write(json.encode(self.settings))
		file:close()
	end
end

function BlackmarketPreviewRotationControl:Load()
	local file = io.open(self._settings_save_path, "r")
	if (file) then
		for k, v in pairs(json.decode(file:read("*all"))) do
			self.settings[k] = v
		end
	else
		self:Save()
	end
	if not self.params then 
		self.params = {
			rotation_enabled = self.settings.default_rotation_enabled,
			yaw_speed = self.settings.yaw_speed,
			roll_speed = self.settings.roll_speed,
			pitch_speed = self.settings.pitch_speed,
			breathing_shaker = nil
		}
	end
end

Hooks:Add("LocalizationManagerPostInit", "bprc_LocalizationManagerPostInit", function( loc )
	if not BeardLib then 
		loc:load_localization_file( BlackmarketPreviewRotationControl._localization_path .. BlackmarketPreviewRotationControl.default_localization_file)
	end
end)

Hooks:Add( "MenuManagerInitialize", "bprc_MenuManagerInitialize", function(menu_manager)


--SETTINGS
	
	MenuCallbackHandler.callback_bprc_enable_default_rotation = function(self,item)
		local value = item:value() == "on"
		if managers.menu_scene then
			--shouldn't actually be necessary since this var is reset each time a new preview is loaded
			--and the player shouldn't be in the menu at that point
			managers.menu_scene._bprc_disable_rotate = not value
		end
		
		BlackmarketPreviewRotationControl.settings.default_rotation_enabled = value
		BlackmarketPreviewRotationControl:Save()
	end
	
	MenuCallbackHandler.callback_bprc_enable_slow_stop = function(self,item)
		BlackmarketPreviewRotationControl.settings.slow_stop = item:value() == "on"
		BlackmarketPreviewRotationControl:Save()
	end
	MenuCallbackHandler.callback_bprc_disable_shaker = function(self,item)
		local value = item:value() == "on"
		BlackmarketPreviewRotationControl.settings.shaker_disabled = value
		if managers.menu_scene then 
			if managers.menu_scene._shaker then 
				if not value then 
					if BlackmarketPreviewRotationControl.params.breathing_shaker then 
						managers.menu_scene._shaker:stop(BlackmarketPreviewRotationControl.params.breathing_shaker)
						BlackmarketPreviewRotationControl.params.breathing_shaker = nil
					else
						managers.menu_scene._shaker:stop_all()
					end
				elseif BlackmarketPreviewRotationControl.params.shaker_disabled then 
					BlackmarketPreviewRotationControl.params.breathing_shaker = managers.menu_scene._shaker:play("breathing",0.2)
				end
			end
		end
		BlackmarketPreviewRotationControl:Save()
	end
	
	MenuCallbackHandler.callback_bprc_set_default_yaw = function(self,item)
		local value = tonumber(item:value())
		BlackmarketPreviewRotationControl.settings.default_yaw = value
		BlackmarketPreviewRotationControl:Save()
	end
	MenuCallbackHandler.callback_bprc_set_default_pitch = function(self,item)
		local value = tonumber(item:value())
		BlackmarketPreviewRotationControl.settings.default_pitch = value
		BlackmarketPreviewRotationControl:Save()
	end
	MenuCallbackHandler.callback_bprc_set_default_roll = function(self,item)
		local value = tonumber(item:value())
		BlackmarketPreviewRotationControl.settings.default_roll = value
		BlackmarketPreviewRotationControl:Save()
	end
	
	MenuCallbackHandler.callback_bprc_set_yaw_speed = function(self,item)
		local value = tonumber(item:value())
		BlackmarketPreviewRotationControl.settings.yaw_speed = value
		BlackmarketPreviewRotationControl.params.yaw_speed = value
		BlackmarketPreviewRotationControl:Save()
	end
	
	--not yet implemented
	MenuCallbackHandler.callback_bprc_set_pitch_speed = function(self,item)
		local value = tonumber(item:value())
		BlackmarketPreviewRotationControl.settings.pitch_speed = value
		BlackmarketPreviewRotationControl.params.pitch_speed = value
		BlackmarketPreviewRotationControl:Save()
	end
	
	--not yet implemented
	MenuCallbackHandler.callback_bprc_set_roll_speed = function(self,item)
		local value = tonumber(item:value())
		BlackmarketPreviewRotationControl.settings.roll_speed = value
		BlackmarketPreviewRotationControl.params.roll_speed = value
		BlackmarketPreviewRotationControl:Save()
	end
	
	MenuCallbackHandler.callback_bprc_set_rotation_acceleration_interval = function(self,item)
		local value = tonumber(item:value())
		BlackmarketPreviewRotationControl.settings.rotation_acceleration_interval = value
		BlackmarketPreviewRotationControl:Save()
	end
	
--KEYBINDS	
	
	MenuCallbackHandler.callback_bprc_keybind_reset_rotation = function(self)
		if managers.menu_scene then 
			local default_rotation = BlackmarketPreviewRotationControl:GetDefaultRotation()
			
			managers.menu_scene._item_yaw = default_rotation:yaw()
			managers.menu_scene._item_pitch = default_rotation:pitch()
			managers.menu_scene._item_roll = default_rotation:roll()
			
--			if BlackmarketPreviewRotationControl.params then 
--				BlackmarketPreviewRotationControl.params.yaw_speed = 0
--			end
		end
	end
	MenuCallbackHandler.callback_bprc_keybind_toggle_rotation = function(self)
		if managers.menu_scene and BlackmarketPreviewRotationControl.params then 
			BlackmarketPreviewRotationControl.params.rotation_enabled = not BlackmarketPreviewRotationControl.params.rotation_enabled
			local enabled = BlackmarketPreviewRotationControl.params.rotation_enabled
			managers.menu_scene._bprc_disable_rotate = not enabled
		end
	end
	MenuCallbackHandler.callback_bprc_keybind_increase_yaw_speed = function(self)
		if BlackmarketPreviewRotationControl.params then 
			BlackmarketPreviewRotationControl.params.yaw_speed = BlackmarketPreviewRotationControl.params.yaw_speed + BlackmarketPreviewRotationControl.settings.rotation_acceleration_interval
		end
	end
	MenuCallbackHandler.callback_bprc_keybind_reset_speed = function(self)
		if BlackmarketPreviewRotationControl.params then 
			BlackmarketPreviewRotationControl.params.yaw_speed = 0
			BlackmarketPreviewRotationControl.params.pitch_speed = 0
			BlackmarketPreviewRotationControl.params.roll_speed = 0
		end
	end
	MenuCallbackHandler.callback_bprc_keybind_decrease_yaw_speed = function(self)
		if BlackmarketPreviewRotationControl.params then 
			BlackmarketPreviewRotationControl.params.yaw_speed = BlackmarketPreviewRotationControl.params.yaw_speed - BlackmarketPreviewRotationControl.settings.rotation_acceleration_interval
		end
	end
	MenuCallbackHandler.callback_bprc_keybind_increase_pitch_speed = function(self)
		if BlackmarketPreviewRotationControl.params then 
			BlackmarketPreviewRotationControl.params.pitch_speed = BlackmarketPreviewRotationControl.params.pitch_speed + BlackmarketPreviewRotationControl.settings.rotation_acceleration_interval
		end
	end
	MenuCallbackHandler.callback_bprc_keybind_decrease_pitch_speed = function(self)
		if BlackmarketPreviewRotationControl.params then 
			BlackmarketPreviewRotationControl.params.pitch_speed = BlackmarketPreviewRotationControl.params.pitch_speed - BlackmarketPreviewRotationControl.settings.rotation_acceleration_interval
		end
	end
	MenuCallbackHandler.callback_bprc_keybind_increase_roll_speed = function(self)
		if BlackmarketPreviewRotationControl.params then 
			BlackmarketPreviewRotationControl.params.roll_speed = BlackmarketPreviewRotationControl.params.roll_speed + BlackmarketPreviewRotationControl.settings.rotation_acceleration_interval
		end
	end
	MenuCallbackHandler.callback_bprc_keybind_decrease_roll_speed = function(self)
		if BlackmarketPreviewRotationControl.params then 
			BlackmarketPreviewRotationControl.params.roll_speed = BlackmarketPreviewRotationControl.params.roll_speed - BlackmarketPreviewRotationControl.settings.rotation_acceleration_interval
		end
	end
	MenuCallbackHandler.callback_bprc_close = function(self)
--		BlackmarketPreviewRotationControl:Save()
	end
	
	BlackmarketPreviewRotationControl:Load()
	
	MenuHelper:LoadFromJsonFile(BlackmarketPreviewRotationControl._settings_path, BlackmarketPreviewRotationControl, BlackmarketPreviewRotationControl.settings)

end)

